#line 2 "macrolexer.c"

#line 4 "macrolexer.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 33
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* %if-c++-only */
/* %endif */

/* %if-c-only */

/* %endif */

/* %if-c-only */

/* %endif */

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* %if-c-only */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
/* %endif */

/* %if-tables-serialization */
/* %endif */
/* end standard C headers. */

/* %if-c-or-c++ */
/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types.
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t;
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;
#endif /* ! C99 */

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! FLEXINT_H */

/* %endif */

/* %if-c++-only */
/* %endif */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else   /* ! __cplusplus */

#if __STDC__

#define YY_USE_CONST

#endif  /* __STDC__ */
#endif  /* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* %not-for-header */

/* Returned upon end-of-file. */
#define YY_NULL 0
/* %ok-for-header */

/* %not-for-header */

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)
/* %ok-for-header */

/* %if-reentrant */

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r

int macrolex_init (yyscan_t* scanner);
/* %endif */

/* %if-not-reentrant */
/* %endif */

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE macrorestart(yyin ,yyscanner )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

/* %if-not-reentrant */
/* %endif */

/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)

/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
    do \
        { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        *yy_cp = yyg->yy_hold_char; \
        YY_RESTORE_YY_MORE_OFFSET \
        yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
        YY_DO_BEFORE_ACTION; /* set up yytext again */ \
        } \
    while ( 0 )

#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef unsigned int yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
    {
/* %if-c-only */
    FILE *yy_input_file;
/* %endif */

/* %if-c++-only */
/* %endif */

    char *yy_ch_buf;        /* input buffer */
    char *yy_buf_pos;       /* current position in input buffer */

    /* Size of input buffer in bytes, not including room for EOB
     * characters.
     */
    yy_size_t yy_buf_size;

    /* Number of characters read into yy_ch_buf, not including EOB
     * characters.
     */
    int yy_n_chars;

    /* Whether we "own" the buffer - i.e., we know we created it,
     * and can realloc() it to grow it, and should free() it to
     * delete it.
     */
    int yy_is_our_buffer;

    /* Whether this is an "interactive" input source; if so, and
     * if we're using stdio for input, then we want to use getc()
     * instead of fread(), to make sure we stop fetching input after
     * each newline.
     */
    int yy_is_interactive;

    /* Whether we're considered to be at the beginning of a line.
     * If so, '^' rules will be active on the next match, otherwise
     * not.
     */
    int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

    /* Whether to try to fill the input buffer when we reach the
     * end of it.
     */
    int yy_fill_buffer;

    int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
    /* When an EOF's been seen but there's still some text to process
     * then we mark the buffer as YY_EOF_PENDING, to indicate that we
     * shouldn't try reading from the input source any more.  We might
     * still have a bunch of tokens to match, though, because of
     * possible backing-up.
     *
     * When we actually see the EOF, we change the status to "new"
     * (via macrorestart()), so that the user can continue scanning by
     * just pointing yyin at a new input file.
     */
#define YY_BUFFER_EOF_PENDING 2

    };
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

/* %if-not-reentrant */
/* %endif */
/* %ok-for-header */

/* %endif */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                          ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]

/* %if-c-only Standard (non-C++) definition */

/* %if-not-reentrant */
/* %not-for-header */

/* %ok-for-header */

/* %endif */

void macrorestart (FILE *input_file ,yyscan_t yyscanner );
void macro_switch_to_buffer (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
YY_BUFFER_STATE macro_create_buffer (FILE *file,int size ,yyscan_t yyscanner );
void macro_delete_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void macro_flush_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void macropush_buffer_state (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
void macropop_buffer_state (yyscan_t yyscanner );

static void macroensure_buffer_stack (yyscan_t yyscanner );
static void macro_load_buffer_state (yyscan_t yyscanner );
static void macro_init_buffer (YY_BUFFER_STATE b,FILE *file ,yyscan_t yyscanner );

#define YY_FLUSH_BUFFER macro_flush_buffer(YY_CURRENT_BUFFER ,yyscanner)

YY_BUFFER_STATE macro_scan_buffer (char *base,yy_size_t size ,yyscan_t yyscanner );
YY_BUFFER_STATE macro_scan_string (yyconst char *yy_str ,yyscan_t yyscanner );
YY_BUFFER_STATE macro_scan_bytes (yyconst char *bytes,int len ,yyscan_t yyscanner );

/* %endif */

void *macroalloc (yy_size_t ,yyscan_t yyscanner );
void *macrorealloc (void *,yy_size_t ,yyscan_t yyscanner );
void macrofree (void * ,yyscan_t yyscanner );

#define yy_new_buffer macro_create_buffer

#define yy_set_interactive(is_interactive) \
    { \
    if ( ! YY_CURRENT_BUFFER ){ \
        macroensure_buffer_stack (yyscanner); \
        YY_CURRENT_BUFFER_LVALUE =    \
            macro_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
    } \
    YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
    }

#define yy_set_bol(at_bol) \
    { \
    if ( ! YY_CURRENT_BUFFER ){\
        macroensure_buffer_stack (yyscanner); \
        YY_CURRENT_BUFFER_LVALUE =    \
            macro_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
    } \
    YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
    }

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* %% [1.0] yytext/yyin/yyout/yy_state_type/yylineno etc. def's & init go here */
/* Begin user sect3 */

#define macrowrap(n) 1
#define YY_SKIP_YYWRAP

#define FLEX_DEBUG

typedef unsigned char YY_CHAR;

typedef int yy_state_type;

#define yytext_ptr yytext_r

/* %if-c-only Standard (non-C++) definition */

static yy_state_type yy_get_previous_state (yyscan_t yyscanner );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  ,yyscan_t yyscanner);
static int yy_get_next_buffer (yyscan_t yyscanner );
static void yy_fatal_error (yyconst char msg[] ,yyscan_t yyscanner );

/* %endif */

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
    yyg->yytext_ptr = yy_bp; \
/* %% [2.0] code to fiddle yytext and yyleng for yymore() goes here \ */\
    yyleng = (size_t) (yy_cp - yy_bp); \
    yyg->yy_hold_char = *yy_cp; \
    *yy_cp = '\0'; \
/* %% [3.0] code to copy yytext_ptr to yytext[] goes here, if %array \ */\
    yyg->yy_c_buf_p = yy_cp;

/* %% [4.0] data tables for the DFA and the user's section 1 definitions go here */
#define YY_NUM_RULES 76
#define YY_END_OF_BUFFER 77
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
    {
    flex_int32_t yy_verify;
    flex_int32_t yy_nxt;
    };
static yyconst flex_int16_t yy_accept[375] =
    {   0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,   77,   75,    7,    8,    7,   75,   75,   75,
       75,   75,   75,   30,   75,   75,   50,   75,   75,   75,
       69,   69,   75,   75,   75,   75,   65,   65,   75,   75,
       75,   75,   75,    4,    7,   75,   75,   75,   75,   75,
       75,   53,   53,   39,   39,   75,   75,    2,    7,   25,
       27,   26,    3,    7,   75,   75,   65,   65,   29,    7,
        1,    7,   29,   29,   29,   28,   29,   29,   29,   29,
       29,   29,   29,   29,   29,   29,   29,    5,    7,   75,

       75,   47,   48,   75,   23,   23,   23,   23,   23,   34,
        7,    6,    7,   34,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   35,   34,   75,   75,
       75,   32,   75,   31,   75,   75,   58,   58,   36,   36,
       33,    8,    8,    8,   72,    0,   64,    0,   45,   45,
       74,    0,    0,   73,    0,   69,    0,    0,    0,   68,
       40,   40,   40,   40,   40,   74,   68,    0,    0,    0,
        0,   66,   72,   72,   65,   71,   43,   63,    4,    4,
        4,    0,   52,    0,    0,    0,    0,   53,   55,   55,
        0,    0,    0,    0,   39,   39,    0,   56,    0,    0,

        2,    2,    2,    3,    3,    3,    0,    0,    0,   67,
       67,   67,   65,   15,   63,    1,    1,    1,    5,    5,
        5,    0,   49,    0,    0,   48,   22,   23,   23,   23,
       23,   23,    6,    6,    6,    0,   57,    0,   45,    0,
        0,   58,   62,   37,   37,   37,   37,   37,   62,    0,
        0,    0,    0,   36,   36,   36,   45,   41,   45,    0,
       46,    0,   44,   44,    0,    0,   40,   40,   40,   40,
       40,   68,   70,    0,   68,   70,   70,   66,   72,   42,
       51,    0,   55,   54,    0,   55,   54,   54,    0,   14,
       63,   16,   67,   67,   67,   22,   18,   19,   20,   23,

       38,    0,   37,   37,   37,   37,   37,   62,   60,    0,
       62,   61,   59,   42,   44,   44,    0,   68,   40,   40,
       40,    0,   55,   16,   67,   67,   23,    0,   62,   37,
       37,   37,   40,    9,   40,   24,   67,   23,   37,    9,
       37,   40,   12,   67,   21,   37,   12,   40,   40,   67,
       37,   37,   10,   40,   67,   10,   37,   40,   67,   67,
       37,   40,   67,   67,   37,   40,   67,   67,   37,   11,
       13,   17,   11,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    2,    4,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    2,    1,    1,    1,    1,
        1,    2,    5,    6,    1,    7,    8,    9,   10,   11,
       12,   13,   14,   15,   16,   17,   18,   19,   20,   21,
       21,   21,   21,   21,   21,   22,   22,   23,    1,   24,
       25,   26,    1,   27,   28,   29,   28,   28,   30,   28,
       31,   31,   32,   31,   31,   31,   31,   32,   33,   32,
       31,   31,   32,   31,   31,   31,   31,   34,   31,   31,
        1,   35,    1,    1,   36,    1,   37,   38,   39,   40,

       41,   28,   42,   31,   43,   31,   31,   44,   45,   46,
       47,   48,   31,   49,   50,   51,   52,   31,   31,   34,
       31,   31,   53,   54,   55,   56,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[57] =
    {   0,
        1,    1,    2,    1,    1,    3,    4,    1,    1,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    5,    5,
        5,    5,    6,    1,    4,    1,    7,    8,    8,    8,
        7,    7,    7,    7,    1,    7,    8,    8,    8,    8,
        8,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    1,    1,    1,    1
    } ;

static yyconst flex_int16_t yy_base[416] =
    {   0,
        0,    0,   55,  110,  166,    0,   55,   56,  220,    0,
      252,    0,  307,  362,  416,  436,  452,    0,  503,  558,
      614,    0, 1299, 1302, 1302,   72, 1292, 1267,   61, 1259,
     1262,   68, 1273, 1302,   65,   66, 1302,   78,  665,   64,
      694,  120,    0,   77,   79,   81,   93,  124,   67,   92,
     1254, 1245, 1244,  127, 1262,  127, 1230, 1251,  135,  226,
      315,  725,  327,    0,  307,  307, 1247,  348, 1246, 1302,
     1302, 1302,  351, 1236, 1199,  376, 1204,  370, 1302, 1302,
      356, 1214, 1184, 1180,   85, 1302,  140, 1170, 1302, 1162,
     1154,  320,  134,  203,  224,  289,  107,  400, 1165,  350,

     1150, 1302,  386,    0,    0, 1110, 1096, 1095, 1079, 1302,
     1302,  407, 1117, 1085, 1034,  113,  356, 1033, 1302, 1023,
     1022,  405,  375,  387,  389,  399, 1302,  316,  419, 1014,
     1019, 1302,  443, 1302,  450,  767,  796,  513, 1001,  504,
     1302,  425,  430,  441, 1302,  441, 1302,    0, 1003,  517,
     1302, 1008,  982, 1302,  525,  568,  442,  974,  995,  658,
        0,  529,  946,  948,  950,  952,  699,  430,  580,  494,
        0,    0,  951,  448,  529, 1302,  541,    0,  575,  589,
      601,  535, 1302,    0,  672,  965,  681,  761,  730,  801,
      490,  717,  587,    0,    0,  685,  575, 1302,    0,  948,

      679,  765,  791,  794,  801,  804,  934,  825,    0,    0,
      909,  906,  918, 1302,  830,  836,  852,  855,  858,  861,
      864,  690, 1302,    0,  930,  850,    0,    0,  879,  883,
      875,  864,  871,  874,  877,  750, 1302,    0,  863,  882,
      868,  874,  878,    0,  890,  795,  793,  798,  901,  654,
      913,  882,    0,  806,  542,  917,  809, 1302,  926,  755,
     1302,    0,    0,  931,  818,  940,    0,  944,  788,  778,
      780,  948,  745,  952,  960,  886,    0,    0,  789, 1302,
      964,  974,  978,  756,  982,  990,  897,    0,  786, 1302,
      994,    0,    0,  744,  740,    0,    0,    0,    0,  727,

     1001, 1011,    0, 1015,  723,  711,  714, 1019,  781, 1023,
     1031,  905,    0, 1302,    0, 1035, 1042, 1046,  704,  702,
      685, 1050, 1054,    0,  685,  677,  676, 1058, 1062,  673,
      671,  649,  643,    0,  642,    0,  629,  569,  545,    0,
      548,  546,  544,  519,    0,  513,  506,  481,  448,  434,
      425,  422,    0,  405,  347,    0,  394,  336,  324,  300,
      284,  267,  203,  201,  183,  174,  105,   64,   33,    0,
        0,    0,    0, 1302, 1084, 1092, 1100, 1108, 1110, 1118,
     1124, 1126, 1130, 1138, 1146, 1150, 1158, 1166, 1168, 1173,
     1177, 1185, 1193, 1195, 1199, 1207, 1215, 1221, 1225, 1229,

     1237, 1239, 1247, 1251, 1255, 1259, 1263, 1267, 1269, 1273,
     1277, 1281, 1285, 1289, 1293
    } ;

static yyconst flex_int16_t yy_def[416] =
    {   0,
      374,    1,  375,  375,  374,    5,    4,    4,    5,    9,
      374,   11,  376,  376,    4,    4,    4,   17,  377,  377,
      374,   21,  374,  374,  374,  374,  374,  374,  378,  379,
      374,  374,  380,  374,  374,  374,  374,  374,  381,  374,
      374,  374,  382,  374,  374,  374,  383,  383,  374,  374,
      374,  374,  374,  374,  374,  384,  374,  385,  374,  374,
      374,  374,  374,  386,  386,  387,  388,  374,  374,  374,
      374,  374,  374,  374,  389,  390,  391,  391,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  392,

      393,  374,  374,  394,  395,  395,  395,  395,  395,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  396,  379,
      397,  374,  374,  374,  374,  398,  374,  374,  399,  399,
      374,  374,  374,  374,  374,  378,  374,  378,  400,  400,
      374,  380,  374,  374,  374,  374,  401,  402,  403,  374,
      404,  404,  404,  404,  404,  374,  374,  374,  374,  374,
      405,  406,  374,  374,  383,  374,  383,   48,  374,  374,
      374,  384,  374,  384,  374,  385,  374,  374,  374,  374,
      374,  374,  374,  407,  386,  386,  387,  374,  387,  388,

      374,  374,  374,  374,  374,  374,  408,  408,  409,  410,
      410,  410,  391,  374,  391,  374,  374,  374,  374,  374,
      374,  392,  374,  392,  393,  374,  411,  395,  395,  395,
      395,  395,  374,  374,  374,  396,  374,  396,  400,  397,
      374,  374,  374,  412,  412,  412,  412,  412,  374,  374,
      374,  374,  413,  399,  399,  399,  400,  374,  400,  401,
      374,  401,  414,  414,  403,  374,  404,  404,  404,  404,
      404,  374,  374,  374,  374,  374,  405,  406,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  407,  408,  374,
      408,  415,  410,  410,  410,  411,  395,  395,  395,  395,

      400,  374,  412,  412,  412,  412,  412,  374,  374,  374,
      374,  374,  413,  374,  414,  414,  374,  374,  404,  404,
      404,  374,  374,  415,  410,  410,  395,  374,  374,  412,
      412,  412,  404,  404,  404,  410,  410,  395,  412,  412,
      412,  404,  404,  410,  395,  412,  412,  404,  404,  410,
      412,  412,  404,  404,  410,  412,  412,  404,  410,  410,
      412,  404,  410,  410,  412,  404,  410,  410,  412,  404,
      410,  410,  412,    0,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,

      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374
    } ;

static yyconst flex_int16_t yy_nxt[1359] =
    {   0,
       24,   25,   26,   27,   28,   29,   30,   31,   32,   33,
       34,   24,   35,   36,   37,   38,   39,   40,   41,   42,
       42,   42,   43,   44,   45,   46,   47,   47,   47,   47,
       47,   48,   47,   47,   24,   47,   47,   47,   47,   47,
       47,   47,   47,   47,   47,   47,   47,   47,   47,   47,
       47,   47,   24,   49,   24,   50,   25,   26,   27,   28,
       66,   66,   31,   32,   67,   67,  147,   35,   51,   37,
       52,   53,   40,  142,  142,  143,  151,  153,   44,   45,
       46,  166,  155,  373,  156,  156,  156,  156,  154,  154,
      154,  151,  151,  151,  155,  148,  156,  156,  156,  156,

      173,  145,  154,  145,  145,  145,  174,  372,   49,  151,
       50,   25,   26,   27,   28,  176,  151,   31,   32,  177,
      151,  151,   35,   51,   37,   52,   53,   40,  179,  179,
      180,  151,  183,   44,   45,   46,  167,  151,  156,  156,
      156,  156,  178,  178,  178,  178,  176,  151,  371,  169,
      177,  187,  153,  188,  188,  188,  188,  173,  145,  154,
      169,  184,  151,   49,  154,   50,   24,   25,   54,   55,
       28,   56,   57,   31,   32,   58,   24,   24,   35,   59,
       37,   60,   61,   40,   62,   63,   63,   63,   24,   44,
       45,   46,   64,   64,   64,   64,   64,   65,   64,   64,

       24,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   24,   49,
       24,   50,   68,   69,  370,   24,   24,  145,  145,   24,
       70,   71,  369,   51,   72,   52,   53,  368,   24,   24,
       24,   24,  187,  367,  188,  188,  188,  188,  145,  174,
      154,   64,   24,   25,   73,   74,   28,   29,   75,   31,
       32,   33,   24,   24,   35,   36,   37,   38,   76,   40,
       41,   42,   42,   42,   43,   44,   45,   46,   77,   77,
       77,   77,   77,   78,   77,   77,   24,   77,   77,   77,
       77,   77,   77,   77,   77,   77,   77,   77,   77,   77,

       77,   77,   77,   77,   24,   49,   24,   50,   80,   81,
       82,   83,  198,  151,   84,   85,  366,   86,   24,   87,
       88,   89,   90,   91,   92,  196,  196,  196,  196,  365,
       93,   94,   95,  189,  189,  189,  189,  166,  364,  154,
      151,  199,  151,  190,  154,  188,  188,  188,  188,  201,
      201,  202,  204,  204,  205,  223,  192,  216,  216,  217,
       96,  363,   97,   80,   81,   82,   83,  192,  153,   84,
       85,  151,   86,   24,   87,   88,   89,   90,   91,   92,
      154,  362,  209,  359,  224,   93,   94,   95,  215,  215,
      215,  215,  214,  360,  160,  160,  160,  160,  173,  145,

      154,  219,  219,  220,  226,  226,  226,  226,  233,  233,
      234,  145,  145,  145,  174,   96,  211,   97,   98,   99,
      212,  100,  166,  151,  237,  101,  142,  142,  143,  154,
      102,  142,  142,  143,  103,  103,  103,  103,   98,   99,
      361,  100,  142,  142,  143,  101,  147,  261,  273,  273,
      102,  358,  151,  238,  103,  103,  103,  103,  104,  241,
      357,  242,  242,  242,  242,  356,  241,  154,  242,  242,
      242,  242,  154,  279,  154,  148,  262,  355,  105,  105,
      105,  105,  105,  105,  105,  105,  354,  105,  105,  105,
      105,  105,  105,  105,  106,  105,  105,  107,  105,  108,

      105,  109,  105,  105,  111,  112,  113,  114,  284,  284,
      115,  116,  276,  276,  276,  117,  118,  119,  120,  121,
      122,  353,  256,  256,  256,  256,  123,  124,  125,  249,
      255,  242,  242,  242,  242,  259,  259,  259,  259,  258,
      183,  352,  251,  160,  160,  160,  160,  268,  268,  268,
      268,  176,  351,  251,  350,  177,  126,  127,  128,  111,
      112,  113,  114,  280,  314,  115,  116,  177,  255,  184,
      117,  118,  119,  120,  121,  122,  179,  179,  180,  349,
      198,  123,  124,  125,  167,  348,  156,  156,  156,  156,
      179,  179,  180,  274,  347,  274,  346,  169,  275,  275,

      275,  275,  179,  179,  180,  287,  287,  287,  169,  199,
      345,  126,  127,  128,   24,   25,   26,   27,   28,  129,
      130,   31,   32,  131,   24,  132,   35,  133,  134,  135,
      136,   40,  137,  138,  138,  138,   24,   44,   45,   46,
      139,  139,  139,  139,  139,  140,  139,  139,   24,  139,
      139,  139,  139,  139,  139,  139,  139,  139,  139,  139,
      139,  139,  139,  139,  139,  139,  141,   49,   24,   50,
      157,  158,  309,  309,  159,  344,  160,  160,  160,  160,
      201,  201,  202,  160,  160,  160,  160,  266,  343,  154,
      281,  281,  281,  281,  342,  223,  162,  341,  266,  189,

      189,  189,  189,  196,  196,  196,  196,  163,  164,  165,
      167,  340,  156,  156,  156,  156,  339,  272,  272,  272,
      272,  338,  168,  169,  224,  337,  170,  171,  266,  336,
      285,  168,  285,  335,  169,  286,  286,  286,  286,  266,
      170,  190,  334,  188,  188,  188,  188,  333,  189,  189,
      189,  189,  332,  191,  192,  237,  331,  193,  194,  282,
      261,  330,  191,  273,  273,  192,  201,  201,  202,  327,
      282,  193,  157,  158,  284,  284,  159,  190,  326,  188,
      188,  188,  188,  325,  238,  243,  243,  243,  243,  262,
      192,  154,  201,  201,  202,  204,  204,  205,  245,  309,

      309,  192,  204,  204,  205,  204,  204,  205,  290,  246,
      247,  248,  249,  154,  242,  242,  242,  242,  321,  283,
      283,  283,  283,  320,  250,  251,  319,  261,  252,  253,
      282,  258,  255,  250,  307,  306,  251,  216,  216,  217,
      305,  282,  252,  291,  291,  291,  291,  290,  215,  215,
      215,  215,  214,  216,  216,  217,  216,  216,  217,  219,
      219,  220,  219,  219,  220,  219,  219,  220,  226,  226,
      226,  226,  233,  233,  234,  233,  233,  234,  233,  233,
      234,  301,  301,  301,  301,  258,  243,  243,  243,  243,
      249,  237,  242,  242,  242,  242,  243,  243,  243,  243,

      312,  312,  312,  251,  276,  276,  276,  302,  304,  304,
      304,  304,  300,  299,  251,  287,  287,  287,  302,  308,
      308,  308,  308,  312,  312,  312,  310,  298,  310,  297,
      302,  311,  311,  311,  311,  256,  256,  256,  256,  223,
      214,  302,  295,  255,  259,  259,  259,  259,  258,  316,
      316,  316,  316,  317,  294,  317,  290,  198,  318,  318,
      318,  318,  268,  268,  268,  268,  272,  272,  272,  272,
      275,  275,  275,  275,  183,  154,  154,  266,  275,  275,
      275,  275,  281,  281,  281,  281,  271,  322,  266,  322,
      270,  269,  323,  323,  323,  323,  283,  283,  283,  283,

      286,  286,  286,  286,  261,  264,  151,  282,  286,  286,
      286,  286,  291,  291,  291,  291,  290,  147,  282,  301,
      301,  301,  301,  258,  328,  258,  328,  255,  237,  329,
      329,  329,  329,  304,  304,  304,  304,  308,  308,  308,
      308,  311,  311,  311,  311,  239,  154,  154,  302,  311,
      311,  311,  311,  316,  316,  316,  316,  154,  151,  302,
      318,  318,  318,  318,  318,  318,  318,  318,  323,  323,
      323,  323,  323,  323,  323,  323,  329,  329,  329,  329,
      329,  329,  329,  329,   24,   24,   24,   24,   24,   24,
       24,   24,   79,   79,   79,   79,   79,   79,   79,   79,

      110,  110,  110,  110,  110,  110,  110,  110,  146,  145,
      146,  146,  146,  146,  146,  146,  149,  149,  152,  235,
      152,  152,  152,  152,  152,  152,  161,  161,  161,  232,
      161,  161,  172,  172,  175,  175,  175,  175,  182,  231,
      182,  182,  182,  182,  182,  182,  186,  230,  186,  186,
      186,  186,  186,  186,  195,  229,  195,  195,  197,  223,
      197,  197,  197,  197,  197,  197,  200,  221,  200,  200,
      200,  200,  200,  200,  207,  207,  210,  210,  154,  210,
      210,  213,  213,  213,  213,  222,  154,  222,  222,  222,
      222,  222,  222,  225,  154,  225,  225,  225,  225,  225,

      225,  227,  227,  228,  151,  228,  228,  236,  145,  236,
      236,  236,  236,  236,  236,  240,  218,  240,  240,  240,
      240,  240,  240,  244,  244,  244,  214,  244,  244,  254,
      208,  254,  254,  257,  257,  257,  257,  260,  206,  260,
      260,  260,  260,  260,  260,  263,  263,  265,  203,  265,
      265,  265,  265,  265,  265,  267,  198,  267,  267,  277,
      183,  185,  277,  278,  181,  278,  278,  288,  154,  154,
      288,  289,  289,  289,  289,  292,  292,  293,  154,  293,
      293,  296,  147,  296,  296,  303,  151,  303,  303,  313,
      150,  145,  313,  315,  144,  315,  315,  324,  374,  324,

      324,   23,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374
    } ;

static yyconst flex_int16_t yy_chk[1359] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    3,    3,    3,    3,
        7,    8,    3,    3,    7,    8,   29,    3,    3,    3,
        3,    3,    3,   26,   26,   26,   32,   35,    3,    3,
        3,   40,   36,  369,   36,   36,   36,   36,   40,   35,
       36,   49,   32,   85,   38,   29,   38,   38,   38,   38,

       44,   44,   38,   45,   45,   46,   46,  368,    3,   85,
        3,    4,    4,    4,    4,   47,   50,    4,    4,   47,
       49,  116,    4,    4,    4,    4,    4,    4,   54,   54,
       54,   97,   56,    4,    4,    4,   42,  116,   42,   42,
       42,   42,   48,   48,   48,   48,   48,   50,  367,   42,
       48,   59,   87,   59,   59,   59,   59,   93,   93,   59,
       42,   56,   97,    4,   87,    4,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,

        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    9,    9,  366,    9,    9,   94,   94,    9,
        9,    9,  365,    9,    9,    9,    9,  364,    9,    9,
        9,    9,   60,  363,   60,   60,   60,   60,   95,   95,
       60,    9,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,

       11,   11,   11,   11,   11,   11,   11,   11,   13,   13,
       13,   13,   66,   96,   13,   13,  362,   13,   13,   13,
       13,   13,   13,   13,   13,   65,   65,   65,   65,  361,
       13,   13,   13,   61,   61,   61,   61,   92,  360,   61,
      128,   66,   96,   63,   92,   63,   63,   63,   63,   68,
       68,   68,   73,   73,   73,  100,   63,   81,   81,   81,
       13,  359,   13,   14,   14,   14,   14,   63,  117,   14,
       14,  128,   14,   14,   14,   14,   14,   14,   14,   14,
      117,  358,   76,  355,  100,   14,   14,   14,   78,   78,
       78,   78,   78,  355,   76,   76,   76,   76,  123,  123,

       76,   98,   98,   98,  103,  103,  103,  103,  112,  112,
      112,  124,  124,  125,  125,   14,   76,   14,   15,   15,
       76,   15,  122,  126,  129,   15,  142,  142,  142,  122,
       15,  143,  143,  143,   15,   15,   15,   15,   16,   16,
      357,   16,  144,  144,  144,   16,  146,  157,  168,  168,
       16,  354,  126,  129,   16,   16,   16,   16,   17,  133,
      352,  133,  133,  133,  133,  351,  135,  133,  135,  135,
      135,  135,  174,  174,  135,  146,  157,  350,   17,   17,
       17,   17,   17,   17,   17,   17,  349,   17,   17,   17,
       17,   17,   17,   17,   17,   17,   17,   17,   17,   17,

       17,   17,   17,   17,   19,   19,   19,   19,  191,  191,
       19,   19,  170,  170,  170,   19,   19,   19,   19,   19,
       19,  348,  140,  140,  140,  140,   19,   19,   19,  138,
      140,  138,  138,  138,  138,  150,  150,  150,  150,  150,
      182,  347,  138,  155,  155,  155,  155,  162,  162,  162,
      162,  175,  346,  138,  344,  175,   19,   19,   19,   20,
       20,   20,   20,  177,  255,   20,   20,  177,  255,  182,
       20,   20,   20,   20,   20,   20,  179,  179,  179,  343,
      197,   20,   20,   20,  156,  342,  156,  156,  156,  156,
      180,  180,  180,  169,  341,  169,  339,  156,  169,  169,

      169,  169,  181,  181,  181,  193,  193,  193,  156,  197,
      338,   20,   20,   20,   21,   21,   21,   21,   21,   21,
       21,   21,   21,   21,   21,   21,   21,   21,   21,   21,
       21,   21,   21,   21,   21,   21,   21,   21,   21,   21,
       21,   21,   21,   21,   21,   21,   21,   21,   21,   21,
       21,   21,   21,   21,   21,   21,   21,   21,   21,   21,
       21,   21,   21,   21,   21,   21,   21,   21,   21,   21,
       39,   39,  250,  250,   39,  337,  160,  160,  160,  160,
      201,  201,  201,   39,   39,   39,   39,  160,  335,   39,
      185,  185,  185,  185,  333,  222,   39,  332,  160,  187,

      187,  187,  187,  196,  196,  196,  196,   39,   39,   39,
       41,  331,   41,   41,   41,   41,  330,  167,  167,  167,
      167,  327,   41,   41,  222,  326,   41,   41,  167,  325,
      192,   41,  192,  321,   41,  192,  192,  192,  192,  167,
       41,   62,  320,   62,   62,   62,   62,  319,  189,  189,
      189,  189,  307,   62,   62,  236,  306,   62,   62,  189,
      260,  305,   62,  273,  273,   62,  202,  202,  202,  300,
      189,   62,  136,  136,  284,  284,  136,  188,  295,  188,
      188,  188,  188,  294,  236,  136,  136,  136,  136,  260,
      188,  136,  203,  203,  203,  204,  204,  204,  136,  309,

      309,  188,  205,  205,  205,  206,  206,  206,  289,  136,
      136,  136,  137,  279,  137,  137,  137,  137,  271,  190,
      190,  190,  190,  270,  137,  137,  269,  265,  137,  137,
      190,  257,  254,  137,  248,  247,  137,  216,  216,  216,
      246,  190,  137,  208,  208,  208,  208,  208,  215,  215,
      215,  215,  215,  217,  217,  217,  218,  218,  218,  219,
      219,  219,  220,  220,  220,  221,  221,  221,  226,  226,
      226,  226,  233,  233,  233,  234,  234,  234,  235,  235,
      235,  239,  239,  239,  239,  239,  241,  241,  241,  241,
      242,  240,  242,  242,  242,  242,  243,  243,  243,  243,

      252,  252,  252,  242,  276,  276,  276,  243,  245,  245,
      245,  245,  232,  231,  242,  287,  287,  287,  243,  249,
      249,  249,  249,  312,  312,  312,  251,  230,  251,  229,
      249,  251,  251,  251,  251,  256,  256,  256,  256,  225,
      213,  249,  212,  256,  259,  259,  259,  259,  259,  264,
      264,  264,  264,  266,  211,  266,  207,  200,  266,  266,
      266,  266,  268,  268,  268,  268,  272,  272,  272,  272,
      274,  274,  274,  274,  186,  173,  166,  272,  275,  275,
      275,  275,  281,  281,  281,  281,  165,  282,  272,  282,
      164,  163,  282,  282,  282,  282,  283,  283,  283,  283,

      285,  285,  285,  285,  159,  158,  153,  283,  286,  286,
      286,  286,  291,  291,  291,  291,  291,  152,  283,  301,
      301,  301,  301,  301,  302,  149,  302,  139,  131,  302,
      302,  302,  302,  304,  304,  304,  304,  308,  308,  308,
      308,  310,  310,  310,  310,  130,  121,  120,  308,  311,
      311,  311,  311,  316,  316,  316,  316,  118,  115,  308,
      317,  317,  317,  317,  318,  318,  318,  318,  322,  322,
      322,  322,  323,  323,  323,  323,  328,  328,  328,  328,
      329,  329,  329,  329,  375,  375,  375,  375,  375,  375,
      375,  375,  376,  376,  376,  376,  376,  376,  376,  376,

      377,  377,  377,  377,  377,  377,  377,  377,  378,  114,
      378,  378,  378,  378,  378,  378,  379,  379,  380,  113,
      380,  380,  380,  380,  380,  380,  381,  381,  381,  109,
      381,  381,  382,  382,  383,  383,  383,  383,  384,  108,
      384,  384,  384,  384,  384,  384,  385,  107,  385,  385,
      385,  385,  385,  385,  386,  106,  386,  386,  387,  101,
      387,  387,  387,  387,  387,  387,  388,   99,  388,  388,
      388,  388,  388,  388,  389,  389,  390,  390,   91,  390,
      390,  391,  391,  391,  391,  392,   90,  392,  392,  392,
      392,  392,  392,  393,   88,  393,  393,  393,  393,  393,

      393,  394,  394,  395,   84,  395,  395,  396,   83,  396,
      396,  396,  396,  396,  396,  397,   82,  397,  397,  397,
      397,  397,  397,  398,  398,  398,   77,  398,  398,  399,
       75,  399,  399,  400,  400,  400,  400,  401,   74,  401,
      401,  401,  401,  401,  401,  402,  402,  403,   69,  403,
      403,  403,  403,  403,  403,  404,   67,  404,  404,  405,
       58,   57,  405,  406,   55,  406,  406,  407,   53,   52,
      407,  408,  408,  408,  408,  409,  409,  410,   51,  410,
      410,  411,   33,  411,  411,  412,   31,  412,  412,  413,
       30,   28,  413,  414,   27,  414,  414,  415,   23,  415,

      415,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374,  374,  374,
      374,  374,  374,  374,  374,  374,  374,  374
    } ;

static yyconst flex_int16_t yy_rule_linenum[76] =
    {   0,
      165,  171,  177,  182,  187,  192,  194,  196,  201,  206,
      211,  216,  221,  225,  230,  240,  245,  250,  254,  258,
      261,  265,  271,  280,  288,  292,  296,  301,  307,  317,
      322,  326,  331,  336,  341,  346,  351,  367,  372,  377,
      401,  416,  424,  431,  445,  460,  468,  472,  479,  484,
      490,  494,  498,  502,  506,  510,  518,  523,  526,  529,
      532,  535,  539,  544,  549,  556,  561,  566,  570,  574,
      581,  586,  590,  594,  604
    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#line 1 "macro.l"
#line 2 "macro.l"

/*
 * $Id: macrolexer.c 23920 2007-12-15 10:50:14Z kjs $
 * Copyright (C) 2007, The Perl Foundation.
 */

/*
 * This pre-processor handles the macro layer of the
 * PIR language. The following constructs are filtered
 * out and handled:
 *
 * .include
 * .macro
 * .macro_local
 * .macro_label
 * .macro_const
 *
 */


#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include "macro.h"
#include "lexer.h"

#define  YY_NO_UNISTD_H
#include "macroparser.h"

#define YY_EXTRA_TYPE  struct lexer_state *


extern macro_def *find_macro(constant_table *table, char *name);
extern char *concat(char *str1, char *str2);



/*

The C89 standard does not define a strdup() in the C library,
so define our own strdup. Function names beginning with "str"
are reserved (I think), so make it dupstr, as that is what it
does: duplicate a string.

*/
char *
dupstr(char const * const source) {
    char *newstring = (char *)calloc(strlen(source) + 1, sizeof (char));
    assert(newstring);
    strcpy(newstring, source);
    return newstring;
}

/*

See dupstr, except that this version takes the number of characters to be
copied. Easy for copying a string except the quotes.

*/
char *
dupstrn(char const * const source, size_t num_chars) {
    char *newstring = (char *)calloc(num_chars + 1, sizeof (char));
    assert(newstring);
    /* only copy num_chars characters */
    strncpy(newstring, source, num_chars);
    return newstring;
}

/*

Update location information. Handling of heredocs is buggy w.r.t. line no.
Fix this routine.

*/
static void
update_location(char *scanned_string, lexer_state *lexer) {
    char *iter = scanned_string;
    while (*iter != '\0') {
        if (*iter == '\n')
            lexer->line++;
        iter++;
    }
}


/*
Use these on a big file with macros, and check out whether there are unused rules.
*/
int ctr[YY_NUM_RULES];
int num_rules = YY_NUM_RULES;


/* after a rule is matched, execute this block of code to keep track of the line no. */
#define YY_USER_ACTION   { ++ctr[yy_act];             \
                           update_location(yytext, macroget_extra(yyscanner)); \
                         }



static int is_pir_directive(char *name);











#line 1070 "macrolexer.c"

#define INITIAL 0
#define MACRODEF 1
#define CONSTDEF 2
#define INCLUDE 3
#define MACROPAR 4
#define MACROBODY 5
#define EXPAND 6
#define LINE 7
#define MACROLOCAL 8
#define BRACEDARGS 9
#define EXPARGS 10

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* %if-c-only */
#include <unistd.h>
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* %if-c-only Reentrant structure and macros (non-C++). */
/* %if-reentrant */

/* Holds the entire state of the reentrant scanner. */
struct yyguts_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE * yy_buffer_stack; /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;

    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;

    YYSTYPE * yylval_r;

    }; /* end struct yyguts_t */

/* %if-c-only */

static int yy_init_globals (yyscan_t yyscanner );

/* %endif */

/* %if-reentrant */

    /* This must go here because YYSTYPE and YYLTYPE are included
     * from bison output in section 1.*/
    #    define yylval yyg->yylval_r

/* %endif */

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int macrolex_destroy (yyscan_t yyscanner );

int macroget_debug (yyscan_t yyscanner );

void macroset_debug (int debug_flag ,yyscan_t yyscanner );

YY_EXTRA_TYPE macroget_extra (yyscan_t yyscanner );

void macroset_extra (YY_EXTRA_TYPE user_defined ,yyscan_t yyscanner );

FILE *macroget_in (yyscan_t yyscanner );

void macroset_in  (FILE * in_str ,yyscan_t yyscanner );

FILE *macroget_out (yyscan_t yyscanner );

void macroset_out  (FILE * out_str ,yyscan_t yyscanner );

int macroget_leng (yyscan_t yyscanner );

char *macroget_text (yyscan_t yyscanner );

int macroget_lineno (yyscan_t yyscanner );

void macroset_lineno (int line_number ,yyscan_t yyscanner );

/* %if-bison-bridge */

YYSTYPE * macroget_lval (yyscan_t yyscanner );

void macroset_lval (YYSTYPE * yylval_param ,yyscan_t yyscanner );

/* %endif */
/* %endif End reentrant structures and macros. */

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int macrowrap (yyscan_t yyscanner );
#else
extern int macrowrap (yyscan_t yyscanner );
#endif
#endif

/* %not-for-header */

/* %ok-for-header */

/* %endif */

#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int ,yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * ,yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT
/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

#ifdef __cplusplus
static int yyinput (yyscan_t yyscanner );
#else
static int input (yyscan_t yyscanner );
#endif
/* %ok-for-header */

/* %endif */
#endif

/* %if-c-only */

    static void yy_push_state (int new_state ,yyscan_t yyscanner);

    static void yy_pop_state (yyscan_t yyscanner );

/* %endif */

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* %if-c-only Standard (non-C++) definition */
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
/* %% [5.0] fread()/read() definition of YY_INPUT goes here unless we're doing C++ \ */\
    if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
        { \
        int c = '*'; \
        size_t n; \
        for ( n = 0; n < max_size && \
                 (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
            buf[n] = (char) c; \
        if ( c == '\n' ) \
            buf[n++] = (char) c; \
        if ( c == EOF && ferror( yyin ) ) \
            YY_FATAL_ERROR( "input in flex scanner failed" ); \
        result = n; \
        } \
    else \
        { \
        errno=0; \
        while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
            { \
            if( errno != EINTR) \
                { \
                YY_FATAL_ERROR( "input in flex scanner failed" ); \
                break; \
                } \
            errno=0; \
            clearerr(yyin); \
            } \
        }\
\
/* %if-c++-only C++ definition \ */\
/* %endif */

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
/* %if-c-only */
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

/* %if-tables-serialization structures and prototypes */
/* %not-for-header */

/* %ok-for-header */

/* %not-for-header */

/* %tables-yydmap generated elements */
/* %endif */
/* end tables serialization structures and prototypes */

/* %ok-for-header */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1
/* %if-c-only Standard (non-C++) definition */

extern int macrolex (YYSTYPE * yylval_param ,yyscan_t yyscanner);

#define YY_DECL int macrolex (YYSTYPE * yylval_param , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

/* %% [6.0] YY_RULE_SETUP definition goes here */
#define YY_RULE_SETUP \
    YY_USER_ACTION

/* %not-for-header */

/** The main scanner function which does all the work.
 */
YY_DECL
{
    register yy_state_type yy_current_state;
    register char *yy_cp, *yy_bp;
    register int yy_act;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [7.0] user's declarations go here */
#line 161 "macro.l"




#line 1378 "macrolexer.c"

    yylval = yylval_param;

    if ( !yyg->yy_init )
        {
        yyg->yy_init = 1;

#ifdef YY_USER_INIT
        YY_USER_INIT;
#endif

        if ( ! yyg->yy_start )
            yyg->yy_start = 1;  /* first start state */

        if ( ! yyin )
/* %if-c-only */
            yyin = stdin;
/* %endif */
/* %if-c++-only */
/* %endif */

        if ( ! yyout )
/* %if-c-only */
            yyout = stdout;
/* %endif */
/* %if-c++-only */
/* %endif */

        if ( ! YY_CURRENT_BUFFER ) {
            macroensure_buffer_stack (yyscanner);
            YY_CURRENT_BUFFER_LVALUE =
                macro_create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
        }

        macro_load_buffer_state(yyscanner );
        }

    while ( 1 )     /* loops until end-of-file is reached */
        {
/* %% [8.0] yymore()-related code goes here */
        yy_cp = yyg->yy_c_buf_p;

        /* Support of yytext. */
        *yy_cp = yyg->yy_hold_char;

        /* yy_bp points to the position in yy_ch_buf of the start of
         * the current run.
         */
        yy_bp = yy_cp;

/* %% [9.0] code to set up and find next match goes here */
        yy_current_state = yyg->yy_start;
yy_match:
        do
            {
            register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
            if ( yy_accept[yy_current_state] )
                {
                yyg->yy_last_accepting_state = yy_current_state;
                yyg->yy_last_accepting_cpos = yy_cp;
                }
            while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
                {
                yy_current_state = (int) yy_def[yy_current_state];
                if ( yy_current_state >= 375 )
                    yy_c = yy_meta[(unsigned int) yy_c];
                }
            yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
            ++yy_cp;
            }
        while ( yy_current_state != 374 );
        yy_cp = yyg->yy_last_accepting_cpos;
        yy_current_state = yyg->yy_last_accepting_state;

yy_find_action:
/* %% [10.0] code to find the action number goes here */
        yy_act = yy_accept[yy_current_state];

        YY_DO_BEFORE_ACTION;

/* %% [11.0] code for yylineno update goes here */

do_action:  /* This label is used only to access EOF actions. */

/* %% [12.0] debug code goes here */
        if ( yy_flex_debug )
            {
            if ( yy_act == 0 )
                fprintf( stderr, "--scanner backing up\n" );
            else if ( yy_act < 76 )
                fprintf( stderr, "--accepting rule at line %ld (\"%s\")\n",
                         (long)yy_rule_linenum[yy_act], yytext );
            else if ( yy_act == 76 )
                fprintf( stderr, "--accepting default rule (\"%s\")\n",
                         yytext );
            else if ( yy_act == 77 )
                fprintf( stderr, "--(end of buffer or a NUL)\n" );
            else
                fprintf( stderr, "--EOF (start condition %d)\n", YY_START );
            }

        switch ( yy_act )
    { /* beginning of action switch */
/* %% [13.0] actions go here */
            case 0: /* must back up */
            /* undo the effects of YY_DO_BEFORE_ACTION */
            *yy_cp = yyg->yy_hold_char;
            yy_cp = yyg->yy_last_accepting_cpos;
            yy_current_state = yyg->yy_last_accepting_state;
            goto yy_find_action;

case 1:
/* rule 1 can match eol */
YY_RULE_SETUP
#line 165 "macro.l"
{ /**/
                      yy_pop_state(yyscanner);
                      yylval->sval = dupstr("\n");
                      return TK_NL;
                    }
    YY_BREAK
case 2:
/* rule 2 can match eol */
YY_RULE_SETUP
#line 171 "macro.l"
{ /* after macro header start the macro body */
                      yy_push_state(MACROBODY, yyscanner);
                      yylval->sval = dupstr("\n");
                      return TK_NL;
                    }
    YY_BREAK
case 3:
/* rule 3 can match eol */
YY_RULE_SETUP
#line 177 "macro.l"
{ /* return newlines in macro body as any tokens */
                       yylval->sval = dupstr("\n");
                       return TK_ANY;
                     }
    YY_BREAK
case 4:
/* rule 4 can match eol */
YY_RULE_SETUP
#line 182 "macro.l"
{ yy_pop_state(yyscanner);
                      yylval->sval = "\n";
                      return TK_NL;
                    }
    YY_BREAK
case 5:
/* rule 5 can match eol */
YY_RULE_SETUP
#line 187 "macro.l"
{ yy_pop_state(yyscanner);
                      yylval->sval = "\n";
                      return TK_NL;
                    }
    YY_BREAK
case 6:
/* rule 6 can match eol */
YY_RULE_SETUP
#line 192 "macro.l"
{ yylval->sval = "\n"; return TK_ANY; }
    YY_BREAK
case 7:
YY_RULE_SETUP
#line 194 "macro.l"
{ /* skip whitespace */ }
    YY_BREAK
case 8:
/* rule 8 can match eol */
YY_RULE_SETUP
#line 196 "macro.l"
{ /* in all other states, return newline as the newline token */
                      yylval->sval = dupstr("\n");
                      return TK_NL;
                    }
    YY_BREAK
case 9:
YY_RULE_SETUP
#line 201 "macro.l"
{ /* */
                      yy_push_state(LINE, yyscanner);
                      return TK_LINE;
                    }
    YY_BREAK
case 10:
YY_RULE_SETUP
#line 206 "macro.l"
{ /* */
                      yy_push_state(INCLUDE, yyscanner);
                      return TK_INCLUDE;
                    }
    YY_BREAK
case 11:
YY_RULE_SETUP
#line 211 "macro.l"
{ /* */
                      yy_push_state(CONSTDEF, yyscanner);
                      return TK_MACRO_CONST;
                    }
    YY_BREAK
case 12:
YY_RULE_SETUP
#line 216 "macro.l"
{ /* */
                      yy_push_state(MACROPAR, yyscanner);
                      return TK_MACRO;
                    }
    YY_BREAK
case 13:
YY_RULE_SETUP
#line 221 "macro.l"
{ /* */
                            return TK_MACRO_LABEL;
                          }
    YY_BREAK
case 14:
YY_RULE_SETUP
#line 225 "macro.l"
{ /* unique label declaration using macro parameter */
                            yylval->sval = dupstr(yytext);
                            return TK_LABEL_ID;
                          }
    YY_BREAK
case 15:
YY_RULE_SETUP
#line 230 "macro.l"
{ /* unique label declaration, add special marker. */
                            yylval->sval = (char *)calloc(yyleng + 1 + 1, sizeof (char));
                            /* stick a special marker "@" so we can recognize this as a label that
                             * must be munged.
                             */
                            strncpy(yylval->sval, yytext, yyleng - 1);
                            strcpy(yylval->sval + yyleng - 1, "@:");
                            return TK_LABEL_ID;
                          }
    YY_BREAK
case 16:
YY_RULE_SETUP
#line 240 "macro.l"
{ /* referring to a declared label or local id. */
                            yylval->sval = dupstr(yytext );
                            return TK_ANY;
                          }
    YY_BREAK
case 17:
YY_RULE_SETUP
#line 245 "macro.l"
{ /* unique local declaration */
                            yy_push_state(MACROLOCAL, yyscanner);
                            return TK_MACRO_LOCAL;
                          }
    YY_BREAK
case 18:
YY_RULE_SETUP
#line 250 "macro.l"
{ yylval->sval = dupstr("int");
                            return TK_INT;
                          }
    YY_BREAK
case 19:
YY_RULE_SETUP
#line 254 "macro.l"
{ yylval->sval = dupstr("num");
                            return TK_NUM;
                          }
    YY_BREAK
case 20:
YY_RULE_SETUP
#line 258 "macro.l"
{ yylval->sval = dupstr("pmc");
                            return TK_PMC;
                          }
    YY_BREAK
case 21:
YY_RULE_SETUP
#line 261 "macro.l"
{ yylval->sval = dupstr("string");
                            return TK_STRING;
                          }
    YY_BREAK
case 22:
YY_RULE_SETUP
#line 265 "macro.l"
{ /* unique local id declaration */
                            yylval->sval = dupstr(yytext);
                            yy_pop_state(yyscanner);
                            return TK_LOCAL_ID;
                          }
    YY_BREAK
case 23:
YY_RULE_SETUP
#line 271 "macro.l"
{ /* unique local id declaration -- non-parameter */
                            yylval->sval = (char *)calloc(yyleng + 1 + 1, sizeof (char));
                            strcpy(yylval->sval, yytext);
                            /* add special marker that this token needs munging later */
                            yylval->sval[yyleng] = '@';
                            yy_pop_state(yyscanner);
                            return TK_LOCAL_ID;
                          }
    YY_BREAK
case 24:
YY_RULE_SETUP
#line 280 "macro.l"
{ /* when reading ".endm" while scanning macro body, go back to previous state */

                     yy_pop_state(yyscanner);
                     yy_pop_state(yyscanner);

                     return TK_ENDM;
                   }
    YY_BREAK
case 25:
YY_RULE_SETUP
#line 288 "macro.l"
{ /* recognize "(" when scanning macro parameter list */
                     return '(';
                   }
    YY_BREAK
case 26:
YY_RULE_SETUP
#line 292 "macro.l"
{ /* recognize "," when scanning macro parameter list */
                     return ',';
                   }
    YY_BREAK
case 27:
YY_RULE_SETUP
#line 296 "macro.l"
{ /* recognize ")" when scanning a macro parameter list */
                     return ')';
                   }
    YY_BREAK
case 28:
YY_RULE_SETUP
#line 301 "macro.l"
{ /* recognize "(" when expanding a macro */
                     yy_push_state(EXPARGS, yyscanner);
                     return '(';
                   }
    YY_BREAK
case 29:
/* rule 29 can match eol */
YY_RULE_SETUP
#line 307 "macro.l"
{ /* anything except a ")" in this state means a constant expansion, which
                      * has no arguments. This works, because if we did see a '(', the
                      * state EXPARGS is activated.
                      */
                     yylval->sval = dupstr(yytext);
                     yy_pop_state(yyscanner);
                     return TK_ANY;
                   }
    YY_BREAK
case 30:
YY_RULE_SETUP
#line 317 "macro.l"
{ /* */
                    yylval->sval = dupstr(yytext);
                    return TK_ANY;
                  }
    YY_BREAK
case 31:
YY_RULE_SETUP
#line 322 "macro.l"
{ /* */
                     return ',';
                   }
    YY_BREAK
case 32:
YY_RULE_SETUP
#line 326 "macro.l"
{ /* */
                     yy_pop_state(yyscanner);
                     return ')';
                   }
    YY_BREAK
case 33:
YY_RULE_SETUP
#line 331 "macro.l"
{ /* braced argument */
                     yy_push_state(BRACEDARGS, yyscanner);
                     return '{';
                   }
    YY_BREAK
case 34:
/* rule 34 can match eol */
YY_RULE_SETUP
#line 336 "macro.l"
{
                     yylval->sval = dupstr(yytext);
                     return TK_ANY;
                   }
    YY_BREAK
case 35:
YY_RULE_SETUP
#line 341 "macro.l"
{
                     yy_pop_state(yyscanner);
                     return '}';
                   }
    YY_BREAK
case 36:
YY_RULE_SETUP
#line 346 "macro.l"
{ /* variable argument of macro expansion */
                     yylval->sval = dupstr(yytext);
                     return TK_IDENT;
                   }
    YY_BREAK
case 37:
YY_RULE_SETUP
#line 351 "macro.l"
{ /* .foo(.bar) */
                      lexer_state *lexer = macroget_extra(yyscanner);
                      macro_def *macro = find_macro(lexer->globaldefinitions, dupstr(yytext + 1));

                      if (macro != NULL) {
                         yylval->sval = macro->body;
                         return TK_IDENT;
                      }
                      else {
                         fprintf(stderr, "Error: cannot find expansion for %s\n", yytext + 1);
                         lexer->errors++;
                      }
                    }
    YY_BREAK
case 38:
YY_RULE_SETUP
#line 367 "macro.l"
{ /* register can be an argument for macro expansion. */
                     yylval->sval = dupstr(yytext);
                     return TK_IDENT;
                   }
    YY_BREAK
case 39:
YY_RULE_SETUP
#line 372 "macro.l"
{ /* constant or macro parameter ID */
                               yylval->sval = dupstr(yytext);
                               return TK_IDENT;
                             }
    YY_BREAK
case 40:
YY_RULE_SETUP
#line 377 "macro.l"
{ /* directive, constant, macro or method-call */
                      if (is_pir_directive(yytext)) {
                          yylval->sval = dupstr(yytext);
                          return TK_ANY;
                      }
                      else { /* not a directive */
                          lexer_state *lexer = macroget_extra(yyscanner);
                          macro_def *macro = find_macro(lexer->globaldefinitions, yytext + 1);
                          if (macro != NULL) { /* it's a macro! */
                              /* only go into EXPAND state if it's a macro/constant */
                              yy_push_state(EXPAND, yyscanner);
                              yylval->mval = macro; /* pass the pointer to the macro_def */
                              return TK_DOT_IDENT;
                          }
                          else { /* it's not a macro, just a method-call, but if it was supposed
                                    to be a macro expansion, we will only see the error in the pir
                                    compiler.
                                  */
                              yylval->sval = dupstr(yytext);
                              return TK_ANY;
                          }
                      }
                   }
    YY_BREAK
case 41:
YY_RULE_SETUP
#line 401 "macro.l"
{ /* */
                     lexer_state *lexer = macroget_extra(yyscanner);
                     macro_def *macro = find_macro(lexer->globaldefinitions, /* skip "$" and ":" */
                                                   dupstrn(yytext + 1, yyleng - 2));

                     if (macro != NULL) { /* found the parameter */
                        yylval->sval = macro->body; /* expand the macro-parameter */
                        return TK_LABEL_EXPANSION;
                     }
                     else {
                        fprintf(stderr, "Error: cannot find expansion for '%s'\n", yytext);
                        lexer->errors++;
                     }
                   }
    YY_BREAK
case 42:
YY_RULE_SETUP
#line 416 "macro.l"
{ /* this label was declared unique by ".macro_label foo:" but was munged so we
                        can recognize it now, and return a signal to the parser it needs to
                        munge it into a unique ID.
                      */
                     yylval->sval = dupstrn(yytext, yyleng - 2);
                     return TK_UNIQUE_LABEL;
                   }
    YY_BREAK
case 43:
YY_RULE_SETUP
#line 424 "macro.l"
{ /* this local was declared unique by ".macro_local", but was munged so we can
                        recognize it now.
                     */
                     yylval->sval = dupstrn(yytext, yyleng - 1);
                     return TK_UNIQUE_LOCAL;
                   }
    YY_BREAK
case 44:
YY_RULE_SETUP
#line 431 "macro.l"
{   /* expanding a declared variable (local or label) */
                     lexer_state *lexer = macroget_extra(yyscanner);
                     macro_def *macro = find_macro(lexer->globaldefinitions, yytext + 2);

                     if (macro != NULL) { /**/
                        yylval->sval = macro->body;
                        return TK_VAR_EXPANSION;
                     }
                     else {
                        fprintf(stderr, "Error: cannot find expansion for symbol '%s'\n", yytext);
                        lexer->errors++;
                     }
                 }
    YY_BREAK
case 45:
YY_RULE_SETUP
#line 445 "macro.l"
{    /* */
                     lexer_state *lexer = macroget_extra(yyscanner);
                     macro_def *macro = find_macro(lexer->globaldefinitions, yytext + 1);
                     if (macro != NULL) { /*  */
                        yylval->sval = macro->body;
                        return TK_VAR_EXPANSION;
                     }
                     else {
                        fprintf(stderr, "Error: cannot find expansion for parameter '%s'\n", yytext + 1);
                        lexer->errors++;
                     }


                   }
    YY_BREAK
case 46:
YY_RULE_SETUP
#line 460 "macro.l"
{/* treat ".'foo'" or ".$P0" as 1 token to prevent the tokens are separated
                        * by a space; they clearly belong to each other if they were written
                        * together (without a space). This is a method-call, life foo .'bar'().
                        */
                       yylval->sval = dupstr(yytext);
                       return TK_ANY;
                      }
    YY_BREAK
case 47:
YY_RULE_SETUP
#line 468 "macro.l"
{ return ','; }
    YY_BREAK
case 48:
YY_RULE_SETUP
#line 472 "macro.l"
{ /*  */
                           yylval->sval = dupstr(yytext);
                           return TK_INTC;
                         }
    YY_BREAK
case 49:
YY_RULE_SETUP
#line 479 "macro.l"
{ /* */
                        yylval->sval = dupstr(yytext);
                        return TK_STRINGC;
                      }
    YY_BREAK
case 50:
YY_RULE_SETUP
#line 484 "macro.l"
{ /* in all other cases (than the above), treat a comma just as any token. */
                        yylval->sval = dupstr(yytext);
                        return TK_ANY;
                      }
    YY_BREAK
case 51:
YY_RULE_SETUP
#line 490 "macro.l"
{ /**/
                       yylval->sval = dupstr(yytext);
                       return TK_IDENT;
                     }
    YY_BREAK
case 52:
YY_RULE_SETUP
#line 494 "macro.l"
{ yylval->sval = dupstr(yytext);
                       return TK_STRINGC;
                     }
    YY_BREAK
case 53:
YY_RULE_SETUP
#line 498 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 54:
YY_RULE_SETUP
#line 502 "macro.l"
{ yylval->sval = dupstr(yytext);
                                return TK_INTC;
                              }
    YY_BREAK
case 55:
YY_RULE_SETUP
#line 506 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_NUMC;
                          }
    YY_BREAK
case 56:
YY_RULE_SETUP
#line 510 "macro.l"
{ /* a quoted string is needed for an .include or the value of a
                             * macro constant. After this token, leave the current state.
                             */
                             yylval->sval = dupstr(yytext);
                             yy_pop_state(yyscanner);
                             return TK_STRINGC;
                          }
    YY_BREAK
case 57:
YY_RULE_SETUP
#line 518 "macro.l"
{ /* quoted strings argument for macro expansion */
                            yylval->sval = dupstr(yytext);
                            return TK_STRINGC;
                          }
    YY_BREAK
case 58:
YY_RULE_SETUP
#line 523 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 59:
YY_RULE_SETUP
#line 526 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 60:
YY_RULE_SETUP
#line 529 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 61:
YY_RULE_SETUP
#line 532 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 62:
YY_RULE_SETUP
#line 535 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_NUMC;
                          }
    YY_BREAK
case 63:
YY_RULE_SETUP
#line 539 "macro.l"
{ /* register */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 64:
YY_RULE_SETUP
#line 544 "macro.l"
{ /* quoted string */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 65:
YY_RULE_SETUP
#line 549 "macro.l"
{ /* identifier */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 66:
YY_RULE_SETUP
#line 556 "macro.l"
{ /* flag */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 67:
YY_RULE_SETUP
#line 561 "macro.l"
{ /* expansions in a macro body; ignore for now. */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 68:
YY_RULE_SETUP
#line 566 "macro.l"
{ yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 69:
YY_RULE_SETUP
#line 570 "macro.l"
{ yylval->sval = dupstr(yytext);
                                     return TK_ANY;
                                   }
    YY_BREAK
case 70:
YY_RULE_SETUP
#line 574 "macro.l"
{ yylval->sval = dupstr(yytext);
                                         return TK_ANY;
                                       }
    YY_BREAK
case 71:
YY_RULE_SETUP
#line 581 "macro.l"
{ /* normal label */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 72:
YY_RULE_SETUP
#line 586 "macro.l"
{ yylval->sval = dupstr(yytext);
                                                          return TK_ANY;
                                                        }
    YY_BREAK
case 73:
YY_RULE_SETUP
#line 590 "macro.l"
{ yylval->sval = dupstr(yytext);
                                                          return TK_ANY;
                                                        }
    YY_BREAK
case 74:
YY_RULE_SETUP
#line 594 "macro.l"
{ yylval->sval = dupstr(yytext);
                                                          return TK_ANY;
                                                        }
    YY_BREAK
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(MACRODEF):
case YY_STATE_EOF(CONSTDEF):
case YY_STATE_EOF(INCLUDE):
case YY_STATE_EOF(MACROPAR):
case YY_STATE_EOF(MACROBODY):
case YY_STATE_EOF(EXPAND):
case YY_STATE_EOF(LINE):
case YY_STATE_EOF(MACROLOCAL):
case YY_STATE_EOF(BRACEDARGS):
case YY_STATE_EOF(EXPARGS):
#line 598 "macro.l"
{ /* when end of file or end of string buffer, stop scanning. */
                      yyterminate();
                    }
    YY_BREAK
case 75:
YY_RULE_SETUP
#line 604 "macro.l"
{ /* just return any single character token we didn't match before. */
                         yylval->sval = dupstr(yytext);
                         return TK_ANY;
                       }
    YY_BREAK
case 76:
YY_RULE_SETUP
#line 609 "macro.l"
ECHO;
    YY_BREAK
#line 2174 "macrolexer.c"

    case YY_END_OF_BUFFER:
        {
        /* Amount of text matched not including the EOB char. */
        int yy_amount_of_matched_text = (int) (yy_cp - yyg->yytext_ptr) - 1;

        /* Undo the effects of YY_DO_BEFORE_ACTION. */
        *yy_cp = yyg->yy_hold_char;
        YY_RESTORE_YY_MORE_OFFSET

        if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
            {
            /* We're scanning a new file or input source.  It's
             * possible that this happened because the user
             * just pointed yyin at a new source and called
             * macrolex().  If so, then we have to assure
             * consistency between YY_CURRENT_BUFFER and our
             * globals.  Here is the right place to do so, because
             * this is the first action (other than possibly a
             * back-up) that will match for the new input source.
             */
            yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
            YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
            YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
            }

        /* Note that here we test for yy_c_buf_p "<=" to the position
         * of the first EOB in the buffer, since yy_c_buf_p will
         * already have been incremented past the NUL character
         * (since all states make transitions on EOB to the
         * end-of-buffer state).  Contrast this with the test
         * in input().
         */
        if ( yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
            { /* This was really a NUL. */
            yy_state_type yy_next_state;

            yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

            yy_current_state = yy_get_previous_state( yyscanner );

            /* Okay, we're now positioned to make the NUL
             * transition.  We couldn't have
             * yy_get_previous_state() go ahead and do it
             * for us because it doesn't know how to deal
             * with the possibility of jamming (and we don't
             * want to build jamming into it because then it
             * will run more slowly).
             */

            yy_next_state = yy_try_NUL_trans( yy_current_state , yyscanner);

            yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

            if ( yy_next_state )
                {
                /* Consume the NUL. */
                yy_cp = ++yyg->yy_c_buf_p;
                yy_current_state = yy_next_state;
                goto yy_match;
                }

            else
                {
/* %% [14.0] code to do back-up for compressed tables and set up yy_cp goes here */
                yy_cp = yyg->yy_last_accepting_cpos;
                yy_current_state = yyg->yy_last_accepting_state;
                goto yy_find_action;
                }
            }

        else switch ( yy_get_next_buffer( yyscanner ) )
            {
            case EOB_ACT_END_OF_FILE:
                {
                yyg->yy_did_buffer_switch_on_eof = 0;

                if ( macrowrap(yyscanner ) )
                    {
                    /* Note: because we've taken care in
                     * yy_get_next_buffer() to have set up
                     * yytext, we can now set up
                     * yy_c_buf_p so that if some total
                     * hoser (like flex itself) wants to
                     * call the scanner after we return the
                     * YY_NULL, it'll still work - another
                     * YY_NULL will get returned.
                     */
                    yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

                    yy_act = YY_STATE_EOF(YY_START);
                    goto do_action;
                    }

                else
                    {
                    if ( ! yyg->yy_did_buffer_switch_on_eof )
                        YY_NEW_FILE;
                    }
                break;
                }

            case EOB_ACT_CONTINUE_SCAN:
                yyg->yy_c_buf_p =
                    yyg->yytext_ptr + yy_amount_of_matched_text;

                yy_current_state = yy_get_previous_state( yyscanner );

                yy_cp = yyg->yy_c_buf_p;
                yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
                goto yy_match;

            case EOB_ACT_LAST_MATCH:
                yyg->yy_c_buf_p =
                &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

                yy_current_state = yy_get_previous_state( yyscanner );

                yy_cp = yyg->yy_c_buf_p;
                yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
                goto yy_find_action;
            }
        break;
        }

    default:
        YY_FATAL_ERROR(
            "fatal flex scanner internal error--no action found" );
    } /* end of action switch */
        } /* end of scanning one token */
} /* end of macrolex */
/* %ok-for-header */

/* %if-c++-only */
/* %not-for-header */

/* %ok-for-header */

/* %endif */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *  EOB_ACT_LAST_MATCH -
 *  EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *  EOB_ACT_END_OF_FILE - end of file
 */
/* %if-c-only */
static int yy_get_next_buffer (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
    register char *source = yyg->yytext_ptr;
    register int number_to_move, i;
    int ret_val;

    if ( yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] )
        YY_FATAL_ERROR(
        "fatal flex scanner internal error--end of buffer missed" );

    if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
        { /* Don't try to fill the buffer, so this is an EOF. */
        if ( yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1 )
            {
            /* We matched a single character, the EOB, so
             * treat this as a final EOF.
             */
            return EOB_ACT_END_OF_FILE;
            }

        else
            {
            /* We matched some text prior to the EOB, first
             * process it.
             */
            return EOB_ACT_LAST_MATCH;
            }
        }

    /* Try to read more data. */

    /* First move last chars to start of buffer. */
    number_to_move = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr) - 1;

    for ( i = 0; i < number_to_move; ++i )
        *(dest++) = *(source++);

    if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
        /* don't do the read, it's not guaranteed to return an EOF,
         * just force an EOF
         */
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;

    else
        {
            int num_to_read =
            YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

        while ( num_to_read <= 0 )
            { /* Not enough room in the buffer - grow it. */

            /* just a shorter name for the current buffer */
            YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

            int yy_c_buf_p_offset =
                (int) (yyg->yy_c_buf_p - b->yy_ch_buf);

            if ( b->yy_is_our_buffer )
                {
                int new_size = b->yy_buf_size * 2;

                if ( new_size <= 0 )
                    b->yy_buf_size += b->yy_buf_size / 8;
                else
                    b->yy_buf_size *= 2;

                b->yy_ch_buf = (char *)
                    /* Include room in for 2 EOB chars. */
                    macrorealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2 ,yyscanner );
                }
            else
                /* Can't grow it, we don't own it. */
                b->yy_ch_buf = 0;

            if ( ! b->yy_ch_buf )
                YY_FATAL_ERROR(
                "fatal error - scanner input buffer overflow" );

            yyg->yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

            num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
                        number_to_move - 1;

            }

        if ( num_to_read > YY_READ_BUF_SIZE )
            num_to_read = YY_READ_BUF_SIZE;

        /* Read in more data. */
        YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
            yyg->yy_n_chars, num_to_read );

        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
        }

    if ( yyg->yy_n_chars == 0 )
        {
        if ( number_to_move == YY_MORE_ADJ )
            {
            ret_val = EOB_ACT_END_OF_FILE;
            macrorestart(yyin  ,yyscanner);
            }

        else
            {
            ret_val = EOB_ACT_LAST_MATCH;
            YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
                YY_BUFFER_EOF_PENDING;
            }
        }

    else
        ret_val = EOB_ACT_CONTINUE_SCAN;

    yyg->yy_n_chars += number_to_move;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

    yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

    return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

/* %if-c-only */
/* %not-for-header */

    static yy_state_type yy_get_previous_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    register yy_state_type yy_current_state;
    register char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [15.0] code to get the start state into yy_current_state goes here */
    yy_current_state = yyg->yy_start;

    for ( yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp )
        {
/* %% [16.0] code to find the next state goes here */
        register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
        if ( yy_accept[yy_current_state] )
            {
            yyg->yy_last_accepting_state = yy_current_state;
            yyg->yy_last_accepting_cpos = yy_cp;
            }
        while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
            {
            yy_current_state = (int) yy_def[yy_current_state];
            if ( yy_current_state >= 375 )
                yy_c = yy_meta[(unsigned int) yy_c];
            }
        yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
        }

    return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *  next_state = yy_try_NUL_trans( current_state );
 */
/* %if-c-only */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    register int yy_is_jam;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner; /* This var may be unused depending upon options. */
/* %% [17.0] code to find the next state, and perhaps do backing up, goes here */
    register char *yy_cp = yyg->yy_c_buf_p;

    register YY_CHAR yy_c = 1;
    if ( yy_accept[yy_current_state] )
        {
        yyg->yy_last_accepting_state = yy_current_state;
        yyg->yy_last_accepting_cpos = yy_cp;
        }
    while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
        {
        yy_current_state = (int) yy_def[yy_current_state];
        if ( yy_current_state >= 375 )
            yy_c = yy_meta[(unsigned int) yy_c];
        }
    yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
    yy_is_jam = (yy_current_state == 374);

    return yy_is_jam ? 0 : yy_current_state;
}

/* %if-c-only */

/* %endif */

/* %if-c-only */
#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (yyscan_t yyscanner)
#else
    static int input  (yyscan_t yyscanner)
#endif

/* %endif */
/* %if-c++-only */
/* %endif */
{
    int c;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    *yyg->yy_c_buf_p = yyg->yy_hold_char;

    if ( *yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
        {
        /* yy_c_buf_p now points to the character we want to return.
         * If this occurs *before* the EOB characters, then it's a
         * valid NUL; if not, then we've hit the end of the buffer.
         */
        if ( yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
            /* This was really a NUL. */
            *yyg->yy_c_buf_p = '\0';

        else
            { /* need more input */
            int offset = yyg->yy_c_buf_p - yyg->yytext_ptr;
            ++yyg->yy_c_buf_p;

            switch ( yy_get_next_buffer( yyscanner ) )
                {
                case EOB_ACT_LAST_MATCH:
                    /* This happens because yy_g_n_b()
                     * sees that we've accumulated a
                     * token and flags that we need to
                     * try matching the token before
                     * proceeding.  But for input(),
                     * there's no matching to consider.
                     * So convert the EOB_ACT_LAST_MATCH
                     * to EOB_ACT_END_OF_FILE.
                     */

                    /* Reset buffer status. */
                    macrorestart(yyin ,yyscanner);

                    /*FALLTHROUGH*/

                case EOB_ACT_END_OF_FILE:
                    {
                    if ( macrowrap(yyscanner ) )
                        return EOF;

                    if ( ! yyg->yy_did_buffer_switch_on_eof )
                        YY_NEW_FILE;
#ifdef __cplusplus
                    return yyinput(yyscanner);
#else
                    return input(yyscanner);
#endif
                    }

                case EOB_ACT_CONTINUE_SCAN:
                    yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
                    break;
                }
            }
        }

    c = *(unsigned char *) yyg->yy_c_buf_p; /* cast for 8-bit char's */
    *yyg->yy_c_buf_p = '\0';    /* preserve yytext */
    yyg->yy_hold_char = *++yyg->yy_c_buf_p;

/* %% [19.0] update BOL and yylineno */

    return c;
}
/* %if-c-only */
#endif  /* ifndef YY_NO_INPUT */
/* %endif */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
/* %if-c-only */
    void macrorestart  (FILE * input_file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    if ( ! YY_CURRENT_BUFFER ){
        macroensure_buffer_stack (yyscanner);
        YY_CURRENT_BUFFER_LVALUE =
            macro_create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
    }

    macro_init_buffer(YY_CURRENT_BUFFER,input_file ,yyscanner);
    macro_load_buffer_state(yyscanner );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void macro_switch_to_buffer  (YY_BUFFER_STATE  new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* TODO. We should be able to replace this entire function body
     * with
     *      macropop_buffer_state();
     *      macropush_buffer_state(new_buffer);
     */
    macroensure_buffer_stack (yyscanner);
    if ( YY_CURRENT_BUFFER == new_buffer )
        return;

    if ( YY_CURRENT_BUFFER )
        {
        /* Flush out information for old buffer. */
        *yyg->yy_c_buf_p = yyg->yy_hold_char;
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
        }

    YY_CURRENT_BUFFER_LVALUE = new_buffer;
    macro_load_buffer_state(yyscanner );

    /* We don't actually know whether we did this switch during
     * EOF (macrowrap()) processing, but the only time this flag
     * is looked at is after macrowrap() is called, so it's safe
     * to go ahead and always set it.
     */
    yyg->yy_did_buffer_switch_on_eof = 1;
}

/* %if-c-only */
static void macro_load_buffer_state  (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
    yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
    yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
    yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
/* %if-c-only */
    YY_BUFFER_STATE macro_create_buffer  (FILE * file, int  size , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    YY_BUFFER_STATE b;

    b = (YY_BUFFER_STATE) macroalloc(sizeof( struct yy_buffer_state ) ,yyscanner );
    if ( ! b )
        YY_FATAL_ERROR( "out of dynamic memory in macro_create_buffer()" );

    b->yy_buf_size = size;

    /* yy_ch_buf has to be 2 characters longer than the size given because
     * we need to put in 2 end-of-buffer characters.
     */
    b->yy_ch_buf = (char *) macroalloc(b->yy_buf_size + 2 ,yyscanner );
    if ( ! b->yy_ch_buf )
        YY_FATAL_ERROR( "out of dynamic memory in macro_create_buffer()" );

    b->yy_is_our_buffer = 1;

    macro_init_buffer(b,file ,yyscanner);

    return b;
}

/** Destroy the buffer.
 * @param b a buffer created with macro_create_buffer()
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void macro_delete_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    if ( ! b )
        return;

    if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
        YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

    if ( b->yy_is_our_buffer )
        macrofree((void *) b->yy_ch_buf ,yyscanner );

    macrofree((void *) b ,yyscanner );
}

/* %if-c-only */

/* %endif */

/* %if-c++-only */
/* %endif */

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a macrorestart() or at EOF.
 */
/* %if-c-only */
    static void macro_init_buffer  (YY_BUFFER_STATE  b, FILE * file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */

{
    int oerrno = errno;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    macro_flush_buffer(b ,yyscanner);

    b->yy_input_file = file;
    b->yy_fill_buffer = 1;

    /* If b is the current buffer, then macro_init_buffer was _probably_
     * called from macrorestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

/* %if-c-only */

        b->yy_is_interactive = 0;

/* %endif */
/* %if-c++-only */
/* %endif */
    errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void macro_flush_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if ( ! b )
        return;

    b->yy_n_chars = 0;

    /* We always need two end-of-buffer characters.  The first causes
     * a transition to the end-of-buffer state.  The second causes
     * a jam in that state.
     */
    b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
    b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

    b->yy_buf_pos = &b->yy_ch_buf[0];

    b->yy_at_bol = 1;
    b->yy_buffer_status = YY_BUFFER_NEW;

    if ( b == YY_CURRENT_BUFFER )
        macro_load_buffer_state(yyscanner );
}

/* %if-c-or-c++ */
/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void macropush_buffer_state (YY_BUFFER_STATE new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if (new_buffer == NULL)
        return;

    macroensure_buffer_stack(yyscanner);

    /* This block is copied from macro_switch_to_buffer. */
    if ( YY_CURRENT_BUFFER )
        {
        /* Flush out information for old buffer. */
        *yyg->yy_c_buf_p = yyg->yy_hold_char;
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
        }

    /* Only push if top exists. Otherwise, replace top. */
    if (YY_CURRENT_BUFFER)
        yyg->yy_buffer_stack_top++;
    YY_CURRENT_BUFFER_LVALUE = new_buffer;

    /* copied from macro_switch_to_buffer. */
    macro_load_buffer_state(yyscanner );
    yyg->yy_did_buffer_switch_on_eof = 1;
}
/* %endif */

/* %if-c-or-c++ */
/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void macropop_buffer_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if (!YY_CURRENT_BUFFER)
        return;

    macro_delete_buffer(YY_CURRENT_BUFFER ,yyscanner);
    YY_CURRENT_BUFFER_LVALUE = NULL;
    if (yyg->yy_buffer_stack_top > 0)
        --yyg->yy_buffer_stack_top;

    if (YY_CURRENT_BUFFER) {
        macro_load_buffer_state(yyscanner );
        yyg->yy_did_buffer_switch_on_eof = 1;
    }
}
/* %endif */

/* %if-c-or-c++ */
/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
/* %if-c-only */
static void macroensure_buffer_stack (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    int num_to_alloc;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    if (!yyg->yy_buffer_stack) {

        /* First allocation is just for 2 elements, since we don't know if this
         * scanner will even need a stack. We use 2 instead of 1 to avoid an
         * immediate realloc on the next call.
         */
        num_to_alloc = 1;
        yyg->yy_buffer_stack = (struct yy_buffer_state**)macroalloc
                                (num_to_alloc * sizeof(struct yy_buffer_state*)
                                , yyscanner);

        memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state*));

        yyg->yy_buffer_stack_max = num_to_alloc;
        yyg->yy_buffer_stack_top = 0;
        return;
    }

    if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1){

        /* Increase the buffer to prepare for a possible push. */
        int grow_size = 8 /* arbitrary grow size */;

        num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
        yyg->yy_buffer_stack = (struct yy_buffer_state**)macrorealloc
                                (yyg->yy_buffer_stack,
                                num_to_alloc * sizeof(struct yy_buffer_state*)
                                , yyscanner);

        /* zero only the new slots.*/
        memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state*));
        yyg->yy_buffer_stack_max = num_to_alloc;
    }
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE macro_scan_buffer  (char * base, yy_size_t  size , yyscan_t yyscanner)
{
    YY_BUFFER_STATE b;

    if ( size < 2 ||
         base[size-2] != YY_END_OF_BUFFER_CHAR ||
         base[size-1] != YY_END_OF_BUFFER_CHAR )
        /* They forgot to leave room for the EOB's. */
        return 0;

    b = (YY_BUFFER_STATE) macroalloc(sizeof( struct yy_buffer_state ) ,yyscanner );
    if ( ! b )
        YY_FATAL_ERROR( "out of dynamic memory in macro_scan_buffer()" );

    b->yy_buf_size = size - 2;  /* "- 2" to take care of EOB's */
    b->yy_buf_pos = b->yy_ch_buf = base;
    b->yy_is_our_buffer = 0;
    b->yy_input_file = 0;
    b->yy_n_chars = b->yy_buf_size;
    b->yy_is_interactive = 0;
    b->yy_at_bol = 1;
    b->yy_fill_buffer = 0;
    b->yy_buffer_status = YY_BUFFER_NEW;

    macro_switch_to_buffer(b ,yyscanner );

    return b;
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan a string. The next call to macrolex() will
 * scan from a @e copy of @a str.
 * @param str a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       macro_scan_bytes() instead.
 */
YY_BUFFER_STATE macro_scan_string (yyconst char * yystr , yyscan_t yyscanner)
{

    return macro_scan_bytes(yystr,strlen(yystr) ,yyscanner);
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan the given bytes. The next call to macrolex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE macro_scan_bytes  (yyconst char * yybytes, int  _yybytes_len , yyscan_t yyscanner)
{
    YY_BUFFER_STATE b;
    char *buf;
    yy_size_t n;
    int i;

    /* Get memory for full buffer, including space for trailing EOB's. */
    n = _yybytes_len + 2;
    buf = (char *) macroalloc(n ,yyscanner );
    if ( ! buf )
        YY_FATAL_ERROR( "out of dynamic memory in macro_scan_bytes()" );

    for ( i = 0; i < _yybytes_len; ++i )
        buf[i] = yybytes[i];

    buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

    b = macro_scan_buffer(buf,n ,yyscanner);
    if ( ! b )
        YY_FATAL_ERROR( "bad buffer in macro_scan_bytes()" );

    /* It's okay to grow etc. this buffer, and we should throw it
     * away when we're done.
     */
    b->yy_is_our_buffer = 1;

    return b;
}
/* %endif */

/* %if-c-only */
    static void yy_push_state (int  new_state , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if ( yyg->yy_start_stack_ptr >= yyg->yy_start_stack_depth )
        {
        yy_size_t new_size;

        yyg->yy_start_stack_depth += YY_START_STACK_INCR;
        new_size = yyg->yy_start_stack_depth * sizeof( int );

        if ( ! yyg->yy_start_stack )
            yyg->yy_start_stack = (int *) macroalloc(new_size ,yyscanner );

        else
            yyg->yy_start_stack = (int *) macrorealloc((void *) yyg->yy_start_stack,new_size ,yyscanner );

        if ( ! yyg->yy_start_stack )
            YY_FATAL_ERROR(
            "out of memory expanding start-condition stack" );
        }

    yyg->yy_start_stack[yyg->yy_start_stack_ptr++] = YY_START;

    BEGIN(new_state);
}

/* %if-c-only */
    static void yy_pop_state  (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if ( --yyg->yy_start_stack_ptr < 0 )
        YY_FATAL_ERROR( "start-condition stack underflow" );

    BEGIN(yyg->yy_start_stack[yyg->yy_start_stack_ptr]);
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

/* %if-c-only */
static void yy_fatal_error (yyconst char* msg , yyscan_t yyscanner)
{
        (void) fprintf( stderr, "%s\n", msg );
    exit( YY_EXIT_FAILURE );
}
/* %endif */
/* %if-c++-only */
/* %endif */

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
    do \
        { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        yytext[yyleng] = yyg->yy_hold_char; \
        yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
        yyg->yy_hold_char = *yyg->yy_c_buf_p; \
        *yyg->yy_c_buf_p = '\0'; \
        yyleng = yyless_macro_arg; \
        } \
    while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/* %if-c-only */
/* %if-reentrant */

/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE macroget_extra  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyextra;
}

/* %endif */

/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int macroget_lineno  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;

    return yylineno;
}

/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int macroget_column  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;

    return yycolumn;
}

/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *macroget_in  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyin;
}

/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *macroget_out  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyout;
}

/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int macroget_leng  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyleng;
}

/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *macroget_text  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yytext;
}

/* %if-reentrant */

/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void macroset_extra (YY_EXTRA_TYPE  user_defined , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyextra = user_defined ;
}

/* %endif */

/** Set the current line number.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void macroset_lineno (int  line_number , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* lineno is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           yy_fatal_error( "macroset_lineno called with no buffer" , yyscanner);

    yylineno = line_number;
}

/** Set the current column.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void macroset_column (int  column_no , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* column is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           yy_fatal_error( "macroset_column called with no buffer" , yyscanner);

    yycolumn = column_no;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see macro_switch_to_buffer
 */
void macroset_in (FILE *  in_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyin = in_str ;
}

void macroset_out (FILE *  out_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyout = out_str ;
}

int macroget_debug  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yy_flex_debug;
}

void macroset_debug (int  bdebug , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yy_flex_debug = bdebug ;
}

/* %endif */

/* %if-reentrant */
/* Accessor methods for yylval and yylloc */

/* %if-bison-bridge */

YYSTYPE * macroget_lval  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yylval;
}

void macroset_lval (YYSTYPE *  yylval_param , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yylval = yylval_param;
}

/* %endif */

/* User-visible API */

/* macrolex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */

int macrolex_init(yyscan_t* ptr_yy_globals)

{
    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) macroalloc ( sizeof( struct yyguts_t ), NULL );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    return yy_init_globals ( *ptr_yy_globals );
}

/* %endif */

/* %if-c-only */
static int yy_init_globals (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from macrolex_destroy(), so don't allocate here.
     */

    yyg->yy_buffer_stack = 0;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = (char *) 0;
    yyg->yy_init = 0;
    yyg->yy_start = 0;

    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = (FILE *) 0;
    yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * macrolex_init()
     */
    return 0;
}
/* %endif */

/* %if-c-only SNIP! this currently causes conflicts with the c++ scanner */
/* macrolex_destroy is for both reentrant and non-reentrant scanners. */
int macrolex_destroy  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* Pop the buffer stack, destroying each element. */
    while(YY_CURRENT_BUFFER){
        macro_delete_buffer(YY_CURRENT_BUFFER ,yyscanner );
        YY_CURRENT_BUFFER_LVALUE = NULL;
        macropop_buffer_state(yyscanner);
    }

    /* Destroy the stack itself. */
    macrofree(yyg->yy_buffer_stack ,yyscanner);
    yyg->yy_buffer_stack = NULL;

    /* Destroy the start condition stack. */
        macrofree(yyg->yy_start_stack ,yyscanner );
        yyg->yy_start_stack = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * macrolex() is called, initialization will occur. */
    yy_init_globals( yyscanner);

/* %if-reentrant */
    /* Destroy the main struct (reentrant only). */
    macrofree ( yyscanner , yyscanner );
    yyscanner = NULL;
/* %endif */
    return 0;
}
/* %endif */

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n , yyscan_t yyscanner)
{
    register int i;
    for ( i = 0; i < n; ++i )
        s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s , yyscan_t yyscanner)
{
    register int n;
    for ( n = 0; s[n]; ++n )
        ;

    return n;
}
#endif

void *macroalloc (yy_size_t  size , yyscan_t yyscanner)
{
    return (void *) malloc( size );
}

void *macrorealloc  (void * ptr, yy_size_t  size , yyscan_t yyscanner)
{
    /* The cast to (char *) in the following accommodates both
     * implementations that use char* generic pointers, and those
     * that use void* generic pointers.  It works with the latter
     * because both ANSI C and C++ allow castless assignment from
     * any pointer type to void*, and deal with argument conversions
     * as though doing an assignment.
     */
    return (void *) realloc( (char *) ptr, size );
}

void macrofree (void * ptr , yyscan_t yyscanner)
{
    free( (char *) ptr );   /* see macrorealloc() for (char *) cast */
}

/* %if-tables-serialization definitions */
/* %define-yytables   The name for this specific scanner's tables. */
#define YYTABLES_NAME "yytables"
/* %endif */

/* %ok-for-header */

#line 609 "macro.l"




/*

=head1 FUNCTIONS

=over 4

=item C<is_pir_directive>

Returns a non-zero value if the specified name is a PIR directive.

*/
static int
is_pir_directive(char *name) {

    /* maybe make this a hash or at least a binary search.
     * Or, make these "special" macros, and have them expand
     * to their own spelling. This would remove the need
     * for special code, such as this.
     */
    static char * const directives[] = {
        ".arg",
        ".begin_call",
        ".begin_return",
        ".begin_yield",
        ".call",
        ".const",
        ".end",
        ".end_call",
        ".end_return",
        ".end_yield",
        ".get_results",
        ".globalconst",
        ".HLL",
        ".HLL_map",
        ".invocant",
        ".lex",
        ".loadlib",
        ".local",
        ".meth_call",
        ".namespace",
        ".nci_call",
        ".param",
        ".return",
        ".sub",
        ".yield",
        NULL /* needed to easily write loops on this array */
    };

    /* iter is a pointer to constant "char *" (strings). */
    char * const *iter = directives;

    while (*iter != NULL) {
        if (strcmp(*iter, name) == 0) {
            return 1;
        }
        iter++;
    }
    return 0;
}

/*

=back

=cut

*/



/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */



